using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;
using System.Diagnostics;

namespace LanguageConfigurationSolution
{
    class LanguageConfigurationEnableXSLTFeatureReceiver : SPFeatureReceiver
    {
        private void ApplyWebConfigModificationsToFarm(SPWebApplication webApp)
        {
            try
            {
                webApp.Farm.Services.GetValue<SPWebService>().ApplyWebConfigModifications();
            }
            catch (System.UnauthorizedAccessException ex)
            {
                SPSecurity.RunWithElevatedPrivileges(delegate
                {
                    EventLog.WriteEntry(this.GetType().FullName, string.Format("Exception occured running the FeatureReceiver when ApplyWebConfigModifications was called.{0}If you give WSS_WPG user account full permission to the config folder the error will disappear!{0}{1}", Environment.NewLine, ex.ToString()), EventLogEntryType.Error);
                });
                throw;
            }
        }

        private string Owner
        {
            get
            {
                return this.GetType().FullName + ", " + this.GetType().Assembly.FullName;
            }
        }

        private const string PAGESNODE = "pages";

        private const string CONTROLSNODE = "controls";

        private SPWebConfigModification AddPagesNode_modification
        {
            get
            {
                SPWebConfigModification modification = new SPWebConfigModification(string.Format("{0}", PAGESNODE), "configuration/system.web");
                modification.Type = SPWebConfigModification.SPWebConfigModificationType.EnsureSection;
                modification.Value = string.Format("<{0} />", PAGESNODE);
                modification.Owner = Owner;
                return modification;
            }
        }

        private SPWebConfigModification AddControlsNode_modification
        {
            get
            {
                SPWebConfigModification modification = new SPWebConfigModification(string.Format("{0}", CONTROLSNODE), string.Format("configuration/system.web/{0}", PAGESNODE));
                modification.Type = SPWebConfigModification.SPWebConfigModificationType.EnsureSection;
                modification.Value = string.Format("<{0} />", CONTROLSNODE);
                modification.Owner = Owner;
                return modification;
            }
        }

        private string TAGPREFIX = "LanguageConfiguration";
        private SPWebConfigModification AddTagPrefix_modification
        {
            get
            {
                Type type = typeof(ConfigurationReader);
                SPWebConfigModification modification = new SPWebConfigModification(string.Format("add[@tagPrefix='{0}']", TAGPREFIX), string.Format("configuration/system.web/{0}/{1}", PAGESNODE, CONTROLSNODE));
                modification.Type = SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode;
                modification.Value = string.Format("<add tagPrefix=\"{0}\" assembly=\"{2}\" namespace=\"{1}\" />", TAGPREFIX, type.Namespace, type.Assembly.FullName);
                modification.Owner = Owner;
                return modification;
            }
        }

        private string SHORTTAGPREFIX = "LC";
        private SPWebConfigModification AddShortTagPrefix_modification
        {
            get
            {
                Type type = typeof(ConfigurationReader);
                SPWebConfigModification modification = new SPWebConfigModification(string.Format("add[@tagPrefix='{0}']", SHORTTAGPREFIX), string.Format("configuration/system.web/{0}/{1}", PAGESNODE, CONTROLSNODE));
                modification.Type = SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode;
                modification.Value = string.Format("<add tagPrefix=\"{0}\" assembly=\"{2}\" namespace=\"{1}\" />", SHORTTAGPREFIX, type.Namespace, type.Assembly.FullName);
                modification.Owner = Owner;
                return modification;
            }
        }


        public override void FeatureActivated(SPFeatureReceiverProperties properties)
        {
            SPWebApplication webapp = properties.Feature.Parent as SPWebApplication;
            if (webapp == null) return;

            webapp.WebConfigModifications.Add(AddPagesNode_modification);
            webapp.WebConfigModifications.Add(AddControlsNode_modification);
            webapp.WebConfigModifications.Add(AddTagPrefix_modification);
            webapp.WebConfigModifications.Add(AddShortTagPrefix_modification);

            webapp.Update(); // Important. Otherwise webconfigmods can be removed upon activating other features
            ApplyWebConfigModificationsToFarm(webapp);
        }

        public override void FeatureDeactivating(SPFeatureReceiverProperties properties)
        {
            SPWebApplication webapp = properties.Feature.Parent as SPWebApplication;
            if (webapp == null) return;

            for (int idx = webapp.WebConfigModifications.Count - 1; idx >= 0; idx--)
            {
                try
                {
                    SPWebConfigModification conmod = webapp.WebConfigModifications[idx];
                    if (string.Compare(conmod.Owner, Owner, true) == 0)
                    {
                        webapp.WebConfigModifications.RemoveAt(idx);
                    }
                }
                catch 
                {
                }
            }

            webapp.Update(); // Important
            ApplyWebConfigModificationsToFarm(webapp);

        }

        public override void FeatureInstalled(SPFeatureReceiverProperties properties)
        {
        }

        public override void FeatureUninstalling(SPFeatureReceiverProperties properties)
        {
        }
    }
}
