using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;
using System.Diagnostics;

namespace LanguageConfigurationSolution
{
    class LanguageConfigurationTagMappingsFeatureReceiver : SPFeatureReceiver
    {
        private void ApplyWebConfigModificationsToFarm(SPWebApplication webApp)
        {
            try
            {
                webApp.Farm.Services.GetValue<SPWebService>().ApplyWebConfigModifications();
            }
            catch (System.UnauthorizedAccessException ex)
            {
                SPSecurity.RunWithElevatedPrivileges(delegate
                {
                    EventLog.WriteEntry(this.GetType().FullName, string.Format("Exception occured running the FeatureReceiver when ApplyWebConfigModifications was called.{0}If you give WSS_WPG user account full permission to the config folder the error will disappear!{0}{1}", Environment.NewLine, ex.ToString()), EventLogEntryType.Error);
                });
                throw;
            }
        }

        private string Owner
        {
            get
            {
                return this.GetType().FullName + ", " + this.GetType().Assembly.FullName;
            }
        }

        private const string PAGESNODE = "pages";

/*        private SPWebConfigModification AddPagesNode_modification
        {
            get
            {
                SPWebConfigModification modification = new SPWebConfigModification(string.Format("{0}", PAGESNODE), "configuration/system.web");
                modification.Type = SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode;
                modification.Value = string.Format("<{0} validateRequest=\"false\"></{0}>", PAGESNODE);
                modification.Owner = Owner;
                return modification;
            }
        }*/

        private const string TAGMAPPINGNODE = "tagMapping";

/*        private SPWebConfigModification AddTagMappingNode_modification
        {
            get
            {
                SPWebConfigModification modification = new SPWebConfigModification(string.Format("{0}", TAGMAPPINGNODE), string.Format("configuration/system.web/{0}", PAGESNODE));
                modification.Type = SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode;
                modification.Value = string.Format("<{0}></{0}>", TAGMAPPINGNODE);
                modification.Owner = Owner;
                return modification;
            }
        }*/

        private SPWebConfigModification AddTagMapping_modification(Type tagType, Type mappedTagType)
        {
                SPWebConfigModification modification = new SPWebConfigModification(string.Format("add[@tagType='{0}']", tagType.FullName ), string.Format("configuration/system.web/{0}/{1}", PAGESNODE, TAGMAPPINGNODE));
                modification.Type = SPWebConfigModification.SPWebConfigModificationType.EnsureChildNode;
                modification.Value = string.Format("<add tagType=\"{0}\" mappedTagType=\"{1}, {2}\" />", tagType.FullName , mappedTagType.FullName, mappedTagType.Assembly.FullName);
                modification.Owner = Owner;
                return modification;
        }


        public override void FeatureActivated(SPFeatureReceiverProperties properties)
        {
            SPWebApplication webapp = properties.Feature.Parent as SPWebApplication;
            if (webapp == null) return;

            //webapp.WebConfigModifications.Add(AddPagesNode_modification);
            //webapp.WebConfigModifications.Add(AddTagMappingNode_modification);
            webapp.WebConfigModifications.Add(AddTagMapping_modification(typeof(Microsoft.SharePoint.Publishing.WebControls.ContentByQueryWebPart) , typeof(LanguageConfigurationSolution.ContentByQueryWebPart) ) );
            webapp.WebConfigModifications.Add(AddTagMapping_modification(typeof(Microsoft.SharePoint.WebPartPages.DataFormWebPart), typeof(LanguageConfigurationSolution.DataFormWebPart)));
            webapp.WebConfigModifications.Add(AddTagMapping_modification(typeof(Microsoft.Office.Server.Search.WebControls.CoreResultsWebPart), typeof(LanguageConfigurationSolution.CoreResultsWebPart)));

            webapp.Update(); // Important. Otherwise webconfigmods can be removed upon activating other features
            ApplyWebConfigModificationsToFarm(webapp);
        }

        public override void FeatureDeactivating(SPFeatureReceiverProperties properties)
        {
            SPWebApplication webapp = properties.Feature.Parent as SPWebApplication;
            if (webapp == null) return;

            for (int idx = webapp.WebConfigModifications.Count - 1; idx >= 0; idx--)
            {
                SPWebConfigModification conmod = webapp.WebConfigModifications[idx];
                if (string.Compare(conmod.Owner, Owner, true) == 0)
                {
                    webapp.WebConfigModifications.RemoveAt(idx);
                }
            }

            webapp.Update(); // Important
            ApplyWebConfigModificationsToFarm(webapp);

        }

        public override void FeatureInstalled(SPFeatureReceiverProperties properties)
        {
        }

        public override void FeatureUninstalling(SPFeatureReceiverProperties properties)
        {
        }
    }
}
