﻿using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.SharePoint.WebControls;

namespace Macaw.Moss2007.ContentChangesOverviewSolution
{
    public class ChangeTypesMenuButton : ToolBarMenuButton, System.Web.UI.IPostBackEventHandler
    {
        [Flags]
        public enum enumChangeTypes
        {
            Add = 1,
            Navigation = 2,
            GroupMembershipAdd = 4,
            RoleDefinitionAdd = 8,
            Delete = 16,
            Restore = 32,
            GroupMembershipDelete = 64,
            RoleDefinitionDelete = 128,
            Move = 256,
            SystemUpdate = 512,
            RoleAssignmentAdd = 1024,
            RoleDefinitionUpdate = 2048,
            Rename = 4096,
            Update = 8192,
            RoleAssignmentDelete = 16384
        }

        public ChangeTypesMenuButton()
        {
            this.ID = "ChangeTypesMenuButton";
        }

        protected override void AddMenuItems()
        {
            MenuItemTemplate miNone = this.AddMenuItem("None", "None", "/_layouts/images/EMPTY.GIF", null, null, "__doPostBack('" + this.UniqueID + "','None')");
            MenuItemTemplate miAll = this.AddMenuItem("All", "All", "/_layouts/images/EMPTY.GIF", null, null, "__doPostBack('" + this.UniqueID + "','All')");
            this.AddMenuItemSeparator();

            foreach (string ChangeTypeKey in Enum.GetNames(typeof(enumChangeTypes)))
            {
                MenuItemTemplate mi = this.AddMenuItem(ChangeTypeKey, ChangeTypeKey, "/_layouts/images/EMPTY.GIF", null, null, null);
                mi.ClientOnClickScript = "__doPostBack('" + this.UniqueID + "','" + ChangeTypeKey + "')";
            }

            base.AddMenuItems();
        }

        protected override void CreateChildControls()
        {
            base.CreateChildControls();


            base.MenuControl.MenuFormat = MenuFormat.ArrowAlwaysVisible;
            if (!this.Page.IsPostBack)
            {
                // Select all MenuItems
                enumChangeTypes value = 0;
                foreach (string ss in Enum.GetNames(typeof(enumChangeTypes)))
                {
                    enumChangeTypes flagvalue = (enumChangeTypes)Enum.Parse(typeof(enumChangeTypes), ss);
                    value |= flagvalue;
                }
                CheckedMenuItemIds = value;
            }
            else
            {
                // Restore state
                CheckedMenuItemIds = CheckedMenuItemIds;
            }
        }

        public enumChangeTypes CheckedMenuItemIds
        {
            get
            {
                EnsureChildControls();
                return (enumChangeTypes)this.ViewState["CheckedMenuItemIds"];
            }
            set
            {
                this.ViewState["CheckedMenuItemIds"] = value;
                string sval = "";
                foreach (string s in Enum.GetNames(typeof(enumChangeTypes)))
                {
                    enumChangeTypes flagval = ((enumChangeTypes)Enum.Parse(typeof(enumChangeTypes), s));
                    if ((value & flagval) == flagval)
                    {
                        if (sval.Length != 0) sval += ",";
                        sval += s;
                    }
                }

                if (string.IsNullOrEmpty(sval))
                {
                    try
                    {
                        base.MenuControl.CheckedMenuItemIds = null;
                    }
                    catch
                    {
                        // probably an exception, because "no-items checked". This is a bug in the MenuControl. This does work.
                    }
                }
                else
                {

                    base.MenuControl.CheckedMenuItemIds = sval;
                }
            }
        }


        #region IPostBackEventHandler Members

        void System.Web.UI.IPostBackEventHandler.RaisePostBackEvent(string eventArgument)
        {
            if (eventArgument.Equals("None"))
            {
                CheckedMenuItemIds = 0;
                return;
            }

            if (eventArgument.Equals("All"))
            {
                enumChangeTypes value = 0;
                foreach (string ss in Enum.GetNames(typeof(enumChangeTypes)))
                {
                    enumChangeTypes flagvalue = (enumChangeTypes)Enum.Parse(typeof(enumChangeTypes), ss);
                    value |= flagvalue;
                }
                CheckedMenuItemIds = value;
                return;
            }

            enumChangeTypes ea = (enumChangeTypes)Enum.Parse(typeof(enumChangeTypes), eventArgument);

            // Toggle
            CheckedMenuItemIds ^= ea;
        }

        #endregion
    }
}
