﻿using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.SharePoint.Administration;
using System.IO;
using System.Diagnostics;
using Microsoft.SharePoint;

namespace Macaw.Moss2007.PublishingScheduleFieldControlAdapterSolution
{
    public class PublishingScheduleFieldControlAdapterJobDefinition : SPJobDefinition
    {

        public const string JobName = "PublishingScheduleFieldControlAdapterJobDefinition";
        private const string APPBROWSER_FOLDER = "App_Browsers";
        public static Guid JobId = new Guid("{8CD2507D-2147-4721-814E-AEE9876BD56D}");
        private const string BROWSERFILENAME = "Macaw.Moss2007.PublishingScheduleFieldControlAdapterSolution.browser";

        public bool Installing
        {
            get
            {
                return (bool)this.Properties["Installing"];
            }
            set
            {
                this.Properties["Installing"] = value;
            }
        }

        public void Init()
        {
            Id = JobId;
            this.Title = "PublishingScheduleFieldControlAdapter Job";
        }

        public PublishingScheduleFieldControlAdapterJobDefinition()
            : base()
        {
            Init();
        }

        public PublishingScheduleFieldControlAdapterJobDefinition(string jobName, SPService service, SPServer server, SPJobLockType targetType)
            : base(jobName, service, server, SPJobLockType.None)
        {
            Init();
        }

        public PublishingScheduleFieldControlAdapterJobDefinition(string jobName, SPWebApplication webApplication)
            : base(jobName, webApplication, null, SPJobLockType.None)
        {
            Init();
        }

        /// <summary>
        /// This method will touch files with the ".browser" extension to force ASP.NET to recompile.
        /// After recompiling the new ControlAdapters will be activated.
        /// </summary>
        /// <param name="path">Path of the folder of the .browser-files</param>
        private void TouchBrowserFiles(string path)
        {
            foreach (string filename in System.IO.Directory.GetFiles(path, "*.browser"))
            {
                System.IO.File.SetLastWriteTime(filename, DateTime.Now);
            }
        }

        /// <summary>
        /// This method will construct a path-string to the App_Browsers folder for the given Zone.
        /// </summary>
        /// <param name="settings">Zone settings</param>
        /// <returns>Physical path for the given zone</returns>
        private string ConstructAppBrowserPath(SPIisSettings settings)
        {
            string basePath = settings.Path.ToString();
            string destPath = Path.Combine(basePath, APPBROWSER_FOLDER);
            return destPath;
        }

        public override void Execute(Guid targetInstanceId)
        {
            SPWebApplication webApplication = this.WebApplication;
            try
            {
                if (Installing)
                {
                    #region Construct the contents for the .browser file
                    System.IO.Stream resource = this.GetType().Assembly.GetManifestResourceStream("Macaw.Moss2007.PublishingScheduleFieldControlAdapterSolution.FeatureCode.PublishingScheduleFieldControlAdapter.browser");
                    if (resource == null || !resource.CanRead)
                    {
                        throw new Exception("Unable to read the resourcefile");
                    }

                    System.IO.StreamReader reader = new System.IO.StreamReader(resource);
                    string browserContents = reader.ReadToEnd();
                    browserContents = browserContents.Replace("{707D9AE2-D892-448f-8DC4-4BBC2C95770D}", typeof(PublishingScheduleFieldControlAdapter).FullName + ", " + typeof(PublishingScheduleFieldControlAdapter).Assembly.FullName);
                    #endregion

                    foreach (SPUrlZone zone in webApplication.IisSettings.Keys)
                    {
                        // The settings of the IIS application to update
                        SPIisSettings settings = webApplication.IisSettings[zone];

                        string destPath = ConstructAppBrowserPath(settings);
                        string fullfilename = Path.Combine(destPath, BROWSERFILENAME);


                        using (System.IO.FileStream file = new System.IO.FileStream(fullfilename, FileMode.Create, FileAccess.Write))
                        {
                            using (System.IO.StreamWriter writer = new System.IO.StreamWriter(file))
                            {
                                writer.Write(browserContents);
                                writer.Flush();
                            }
                        }

                        TouchBrowserFiles(destPath);
                    }
                }
                else
                {
                    foreach (SPUrlZone zone in webApplication.IisSettings.Keys)
                    {
                        // The settings of the IIS application to update
                        SPIisSettings settings = webApplication.IisSettings[zone];

                        string destPath = ConstructAppBrowserPath(settings);
                        string fullfilename = Path.Combine(destPath, BROWSERFILENAME);

                        if (System.IO.File.Exists(fullfilename))
                        {
                            System.IO.File.Delete(fullfilename);
                        }
                        else
                        {
                            // File not present, thus not deleted
                        }

                        TouchBrowserFiles(destPath);
                    }
                }
            }
            catch (Exception)
            {
            }
            finally
            {
                base.Execute(targetInstanceId);
            }
        }

    }
}
