﻿using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.SharePoint.Administration;
using System.IO;
using System.Diagnostics;
using Microsoft.SharePoint;

namespace Macaw.Moss2007.TimerJobServerSelectorSolution
{
    public class JobEditControlAdapterJobDefinition : SPJobDefinition
    {

        public const string JobName = "JobEditControlAdapterJob";
        private const string BROWSERFILENAME = "WebPartZonMacaw.Moss2007.JobEditControlAdaptereAdapter.browser";
        private const string APPBROWSER_FOLDER = "App_Browsers";

        public bool Installing
        {
            get
            {
                return (bool)this.Properties["Installing"];
            }
            set
            {
                this.Properties["Installing"] = value;
            }
        }

        public JobEditControlAdapterJobDefinition()
            : base()
        {
        }

        public JobEditControlAdapterJobDefinition(string jobName, SPService service, SPServer server, SPJobLockType targetType)
            : base(jobName, service, server, SPJobLockType.None)
        {
        }
        public JobEditControlAdapterJobDefinition(string jobName, SPWebApplication webApplication)
            : base(jobName, webApplication, null, SPJobLockType.None)
        {
            this.Title = "JobEditControlAdapter Job";
        }

        /// <summary>
        /// This method will touch files with the ".browser" extension to force ASP.NET to recompile.
        /// After recompiling the new ControlAdapters will be activated.
        /// </summary>
        /// <param name="path">Path of the folder of the .browser-files</param>
        private void TouchBrowserFiles(string path)
        {
            foreach (string filename in System.IO.Directory.GetFiles(path, "*.browser"))
            {
                System.IO.File.SetLastWriteTime(filename, DateTime.Now);
            }
        }

        /// <summary>
        /// This method will construct a path-string to the App_Browsers folder for the given Zone.
        /// </summary>
        /// <param name="settings">Zone settings</param>
        /// <returns>Physical path for the given zone</returns>
        private string ConstructAppBrowserPath(SPIisSettings settings)
        {
            string basePath = settings.Path.ToString();
            string destPath = Path.Combine(basePath, APPBROWSER_FOLDER);
            return destPath;
        }



        public override void Execute(Guid targetInstanceId)
        {
            try
            {
                SPWebApplication webApplication = this.WebApplication;
                if (Installing)
                {
                    foreach (KeyValuePair<SPUrlZone, SPIisSettings> settingspair in webApplication.IisSettings)
                    {
                        string filename = "Microsoft.SharePoint.ApplicationPages.Administration.dll";
                        string sourcepath = Path.Combine(settingspair.Value.Path.FullName, "_app_bin");
                        string destpath = Path.Combine(settingspair.Value.Path.FullName, "bin");
                        File.Copy(Path.Combine(sourcepath, filename), Path.Combine(destpath, filename), true);
                    }

                    #region Construct the contents for the .browser file
                    System.IO.Stream resource = this.GetType().Assembly.GetManifestResourceStream("Macaw.Moss2007.TimerJobServerSelectorSolution.FeatureCode.Macaw.Moss2007.JobEditControlAdapter.browser");
                    if (resource == null || !resource.CanRead)
                    {
                        throw new Exception("Unable to read the resourcefile");
                    }

                    System.IO.StreamReader reader = new System.IO.StreamReader(resource);
                    string browserContents = reader.ReadToEnd();
                    browserContents = browserContents.Replace("{707D9AE2-D892-448f-8DC4-4BBC2C95770D}", typeof(JobEditControlAdapter).FullName + ", " + typeof(JobEditControlAdapter).Assembly.FullName);
                    #endregion

                    foreach (SPUrlZone zone in webApplication.IisSettings.Keys)
                    {
                        // The settings of the IIS application to update
                        SPIisSettings settings = webApplication.IisSettings[zone];

                        string destPath = ConstructAppBrowserPath(settings);
                        string fullfilename = Path.Combine(destPath, BROWSERFILENAME);

                        Trace.WriteLine(string.Format("{1}: Writing the file \"{0}\"", fullfilename, this.GetType().FullName));

                        using (System.IO.FileStream file = new System.IO.FileStream(fullfilename, FileMode.Create, FileAccess.Write))
                        {
                            using (System.IO.StreamWriter writer = new System.IO.StreamWriter(file))
                            {
                                writer.Write(browserContents);
                                writer.Flush();
                            }
                        }

                        TouchBrowserFiles(destPath);
                    }
                }
                else
                {
                    foreach (KeyValuePair<SPUrlZone, SPIisSettings> settingspair in webApplication.IisSettings)
                    {
                        string filename = "Microsoft.SharePoint.ApplicationPages.Administration.dll";
                        string destpath = Path.Combine(settingspair.Value.Path.FullName, "bin");
                        File.Delete(Path.Combine(destpath, filename));
                    }

                    foreach (SPUrlZone zone in webApplication.IisSettings.Keys)
                    {
                        // The settings of the IIS application to update
                        SPIisSettings settings = webApplication.IisSettings[zone];

                        string destPath = ConstructAppBrowserPath(settings);
                        string fullfilename = Path.Combine(destPath, BROWSERFILENAME);

                        if (System.IO.File.Exists(fullfilename))
                        {
                            Trace.WriteLine(string.Format("{1}: Removing the file \"{0}\"", fullfilename, this.GetType().FullName));
                            System.IO.File.Delete(fullfilename);
                        }
                        else
                        {
                            Trace.WriteLine(string.Format("{1}: File \"{0}\" is not present, thus not deleted.", fullfilename, this.GetType().FullName));
                        }

                        TouchBrowserFiles(destPath);
                    }
                }
            }
            catch (Exception ex)
            {
                Trace.WriteLine(string.Format("{1}: Error running the Job.\r\n{0}", ex.ToString(), this.GetType().FullName));

                throw;
            }
            base.Execute(targetInstanceId);
        }

    }
}
