using System;
using System.Runtime.InteropServices;
using System.Security.Permissions;

using System.IO;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using Microsoft.SharePoint;
using Microsoft.SharePoint.WebControls;
using System.ComponentModel;
using System.Configuration;
using System.Diagnostics;
using System.Threading;
using System.Collections.Generic;

namespace CaptchaFieldControlSolution
{
    public class CaptchaField : SPFieldText
    {
        private const string THREADDATANAME_RECAPTCHAPUBLICKEY = "RecaptchaPublicKey";
        private const string THREADDATANAME_RECAPTCHAPRIVATEKEY = "RecaptchaPrivateKey";
        private const string THREADDATANAME_RECAPTCHATHEME = "RecaptchaTheme";

        #region Private fields
        private string _recaptchapublickey;
        private string _recaptchaprivatekey;
        private string _recaptchatheme;
        private bool _preventUpdate = false;
        #endregion

        #region Constructors
        public CaptchaField(SPFieldCollection fields, string fieldName)
            : base(fields, fieldName)
        {
        }

        public CaptchaField(SPFieldCollection fields, string typeName, string displayName)
            : base(fields, typeName, displayName)
        {
        }
        #endregion

        #region Methods used for temporary storage
        /// <summary>
        /// http://msdn.microsoft.com/en-us/library/cc889345.aspx
        /// </summary>
        /// <param name="propertyName"></param>
        /// <param name="value"></param>
        protected void SetNamedThreadData(string propertyName, object value)
        {
            Thread.SetData(Thread.GetNamedDataSlot(propertyName), value);
        }

        protected void ClearNamedThreadData(string propertyName)
        {
            Thread.FreeNamedDataSlot(propertyName);
        }

        protected object GetNamedThreadData(string propertyName)
        {
            return Thread.GetData(Thread.GetNamedDataSlot(propertyName));
        }
        #endregion

        #region Public Properties
        /// <summary>
        /// Returns an specialized FieldControl for this Field/FieldType
        /// </summary>
        public override Microsoft.SharePoint.WebControls.BaseFieldControl FieldRenderingControl
        {
            get
            {
                BaseFieldControl control = new CaptchaFieldControl();
                control.FieldName = this.InternalName;
                return control;
            }
        }

        public string RecaptchaTheme
        {
            get
            {
                if (_recaptchatheme == null)
                {
                    string value = (string)GetCustomProperty("RecaptchaTheme");
                    _recaptchatheme = (value != null) ? value : null;
                }

                return _recaptchatheme;
            }
            set
            {
                string fv = null;
                if (!string.IsNullOrEmpty(value)) fv = value;
                _recaptchatheme = value;
                SetNamedThreadData(THREADDATANAME_RECAPTCHATHEME, fv);
            }
        }
        
        public string RecaptchaPublicKey
        {
            get
            {
                if (_recaptchapublickey == null)
                {
                    string value = (string)GetCustomProperty("RecaptchaPublicKey");
                    _recaptchapublickey = (value != null) ? value : null;
                }

                return _recaptchapublickey;
            }
            set
            {
                string fv = null;
                if (!string.IsNullOrEmpty(value)) fv = value;
                _recaptchapublickey = value;
                SetNamedThreadData(THREADDATANAME_RECAPTCHAPUBLICKEY, fv);
            }
        }

        public string RecaptchaPrivateKey
        {
            get
            {
                if (_recaptchaprivatekey == null)
                {
                    string value = (string)GetCustomProperty("RecaptchaPrivateKey");
                    _recaptchaprivatekey = (value != null) ? value : null;
                }

                return _recaptchaprivatekey;
            }
            set
            {
                string fv = null;
                if (!string.IsNullOrEmpty(value)) fv = value;
                _recaptchaprivatekey = value;
                SetNamedThreadData(THREADDATANAME_RECAPTCHAPRIVATEKEY, fv);
            }
        }
        #endregion

        #region Public methods
        /// <summary>
        /// override the OnAdded method to call Update
        /// because Update is not called when creating new fields 
        /// </summary>
        public override void OnAdded(SPAddFieldOptions op)
        {
            _preventUpdate = true;
            string recaptchapublickey = (string)GetNamedThreadData(THREADDATANAME_RECAPTCHAPUBLICKEY);
            SetCustomProperty("RecaptchaPublicKey", recaptchapublickey);

            string recaptchaprivatekey = (string)GetNamedThreadData(THREADDATANAME_RECAPTCHAPRIVATEKEY);
            SetCustomProperty("RecaptchaPrivateKey", recaptchaprivatekey);

            string recaptchatheme = (string)GetNamedThreadData(THREADDATANAME_RECAPTCHATHEME);
            SetCustomProperty("RecaptchaTheme", recaptchatheme);

            Update();
            ClearNamedThreadData(THREADDATANAME_RECAPTCHAPUBLICKEY);
            ClearNamedThreadData(THREADDATANAME_RECAPTCHAPRIVATEKEY);
            ClearNamedThreadData(THREADDATANAME_RECAPTCHATHEME);
        }

        public override void OnUpdated()
        {
            if (_preventUpdate == false)
            {
                _preventUpdate = true;

                string recaptchapublickey = (string)GetNamedThreadData(THREADDATANAME_RECAPTCHAPUBLICKEY);
                SetCustomProperty("RecaptchaPublicKey", recaptchapublickey);

                string recaptchaprivatekey = (string)GetNamedThreadData(THREADDATANAME_RECAPTCHAPRIVATEKEY);
                SetCustomProperty("RecaptchaPrivateKey", recaptchaprivatekey);

                string recaptchatheme = (string)GetNamedThreadData(THREADDATANAME_RECAPTCHATHEME);
                SetCustomProperty("RecaptchaTheme", recaptchatheme);
                Update();
                ClearNamedThreadData(THREADDATANAME_RECAPTCHAPUBLICKEY);
                ClearNamedThreadData(THREADDATANAME_RECAPTCHAPRIVATEKEY);
                ClearNamedThreadData(THREADDATANAME_RECAPTCHATHEME);
            }
        }

        public override object GetFieldValue(string value)
        {
            return value;
        }

        #endregion
    }
}
