﻿using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.SharePoint.Administration;
using System.IO;
using System.Diagnostics;
using Microsoft.SharePoint;

namespace Macaw.Wss3.DateFormatModifySolution
{
    public class DateTimeControlAdapterJobDefinition : SPJobDefinition
    {

        public const string JobName = "DateTimeControlAdapterJob";
        private const string BROWSERFILENAME = "DateTimeControlAdapter.browser";
        private const string APPBROWSER_FOLDER = "App_Browsers";

        public bool Installing
        {
            get
            {
                return (bool)this.Properties["Installing"];
            }
            set
            {
                this.Properties["Installing"] = value;
            }
        }

        public DateTimeControlAdapterJobDefinition()
            : base()
        {
        }

        public DateTimeControlAdapterJobDefinition(string jobName, SPService service, SPServer server, SPJobLockType targetType)
            : base(jobName, service, server, SPJobLockType.None)
        {
        }
        public DateTimeControlAdapterJobDefinition(string jobName, SPWebApplication webApplication)
            : base(jobName, webApplication, null, SPJobLockType.None)
        {
            this.Title = "DateTimeControlAdapter Job";
        }

        /// <summary>
        /// This method will touch files with the ".browser" extension to force ASP.NET to recompile.
        /// After recompiling the new ControlAdapters will be activated.
        /// </summary>
        /// <param name="path">Path of the folder of the .browser-files</param>
        private void TouchBrowserFiles(string path)
        {
            foreach (string filename in System.IO.Directory.GetFiles(path, "*.browser"))
            {
                System.IO.File.SetLastWriteTime(filename, DateTime.Now);
            }
        }

        /// <summary>
        /// This method will construct a path-string to the App_Browsers folder for the given Zone.
        /// </summary>
        /// <param name="settings">Zone settings</param>
        /// <returns>Physical path for the given zone</returns>
        private string ConstructAppBrowserPath(SPIisSettings settings)
        {
            string basePath = settings.Path.ToString();
            string destPath = Path.Combine(basePath, APPBROWSER_FOLDER);
            return destPath;
        }



        public override void Execute(Guid targetInstanceId)
        {
            Trace.WriteLine(string.Format("{0}: Execute - start", this.GetType().FullName ));
            try
            {
                SPWebApplication webApplication = this.WebApplication;
                if (Installing)
                {
                    #region Construct the contents for the .browser file
                    System.IO.Stream resource = this.GetType().Assembly.GetManifestResourceStream("Macaw.Wss3.DateFormatModifySolution.FeatureCode.DateTimeControlAdapter.browser");
                    if (resource == null || !resource.CanRead)
                    {
                        throw new Exception("Unable to read the resourcefile");
                    }

                    System.IO.StreamReader reader = new System.IO.StreamReader(resource);
                    string browserContents = reader.ReadToEnd();
                    browserContents = browserContents.Replace("{0D9D13A3-1146-4fcb-8C04-C9C163A3F4E4}", typeof(DateTimeControlAdapter).FullName + ", " + typeof(DateTimeControlAdapter).Assembly.FullName);
                    #endregion

                    foreach (SPUrlZone zone in webApplication.IisSettings.Keys)
                    {
                        // The settings of the IIS application to update
                        SPIisSettings settings = webApplication.IisSettings[zone];

                        string destPath = ConstructAppBrowserPath(settings);
                        string fullfilename = Path.Combine(destPath, BROWSERFILENAME);

                        Trace.WriteLine(string.Format("{0}: Writing the file \"{1}\"" , this.GetType().FullName , fullfilename));

                        using (System.IO.FileStream file = new System.IO.FileStream(fullfilename, FileMode.Create, FileAccess.Write))
                        {
                            using (System.IO.StreamWriter writer = new System.IO.StreamWriter(file))
                            {
                                writer.Write(browserContents);
                                writer.Flush();
                            }
                        }

                        TouchBrowserFiles(destPath);
                    }
                }
                else
                {
                    foreach (SPUrlZone zone in webApplication.IisSettings.Keys)
                    {
                        // The settings of the IIS application to update
                        SPIisSettings settings = webApplication.IisSettings[zone];

                        string destPath = ConstructAppBrowserPath(settings);
                        string fullfilename = Path.Combine(destPath, BROWSERFILENAME);

                        if (System.IO.File.Exists(fullfilename))
                        {
                            Trace.WriteLine(string.Format("{0}: Removing the file \"{1}\"" , this.GetType().FullName, fullfilename));
                            
                            System.IO.File.Delete(fullfilename);
                        }
                        else
                        {
                            Trace.WriteLine(string.Format("{0}: File \"{1}\" is not present, thus not deleted." , this.GetType().FullName, fullfilename));
                        }

                        TouchBrowserFiles(destPath);
                    }
                }
            }
            catch (Exception ex)
            {
                Trace.WriteLine(string.Format("{0}: Error!\r\n{1}", this.GetType().FullName , ex.ToString() ));                
            }
            base.Execute(targetInstanceId);
            Trace.WriteLine(string.Format("{0}: Execute - end", this.GetType().FullName) );
        }

    }
}
